program test
   use omp_lib

   implicit none

   interface
      double precision function gettime() bind(c)
         use iso_c_binding
      end function gettime
   end interface

   double precision, dimension(:), allocatable :: a, b
   integer :: e, n, i, p, nrep, rep
   double precision :: t1, t2
   character(80) :: arg
   character(80) :: benchmark = 'copy'
   integer :: icopy = 0, iscale = 0

   if( command_argument_count() /= 3 )then
      write(*,*) 'Usage: benchmark e nrep'
      stop
   end if

   call get_command_argument(1, benchmark)
   call get_command_argument(2, arg)
   read(arg,'(i8)') e
   call get_command_argument(3, arg)
   read(arg,'(i8)') nrep

   if( benchmark == 'copy' ) icopy = 1
   if( benchmark == 'scale' ) iscale = 1

   n = 2**e

!$omp parallel
!$omp master
   p = omp_get_num_threads()
!$omp end master
!$omp end parallel 

   allocate(a(n), b(n))

   do rep = 1, nrep
      if( benchmark == 'copy' )then
         do i = 1, n 
            a(i) = mod(i, 3)
            b(i) = mod(i, 5)
         end do
      elseif( benchmark == 'scale') then
         do i = 1, n 
            a(i) = mod(i, 3)
         end do
      end if
      if( benchmark == 'copy' )then
         t1 = gettime()
         call pcopy(n, a, b)
         t2 = gettime()
      elseif( benchmark == 'scale' )then
         t1 = gettime()
         call pscale(n, a)
         t2 = gettime()
      end if
      write(*,'(a2,6a10)') '# ', 'nth', 'n', 'Tp', 'Tp/el', 'copy?', 'scale?'
      write(*,'(a2,2i10,f10.6,e10.3,2i10)') '', p, n, t2 - t1, (t2 - t1) / n, icopy, iscale
      do i = 1, n
         if( benchmark == 'copy' )then
            if( b(i) /= a(i) )then
               write(*,*) '!!! failed to copy a to b !!!'
               exit
            end if
         elseif( benchmark == 'scale' )then
            if( a(i) /= 2 * mod(i, 3) )then
               write(*,*) '!!! failed to scale a !!!'
               exit
            end if
         end if
      end do
   end do

   deallocate(a, b)

contains

subroutine pscale(n, a)
   implicit none

   integer :: n
   double precision, dimension(n) :: a

   integer :: i

!$omp parallel 
!$omp do
   do i = 1, n
      a(i) = a(i) * 2
   end do
!$omp end do
!$omp end parallel
end subroutine pscale

subroutine pcopy(n, a, b)
   implicit none

   integer :: n
   double precision, dimension(n) :: a, b

   integer :: i

!$omp parallel 
!$omp do
   do i = 1, n 
      b(i) = a(i)
   end do
!$omp end do 
!$omp end parallel

end subroutine pcopy

end program test
